import os
import glob
import logging
import geckodriver_autoinstaller

import srt_deepl.deepl

from srt_deepl.srt_parser import wrap_line, save_srt

# Check if the current version of geckodriver exists
geckodriver_autoinstaller.install()

INPUT_LANG = {
    "auto": "Any language (detect)",
    "bg": "Bulgarian",
    "zh": "Chinese",
    "cs": "Czech",
    "da": "Danish",
    "nl": "Dutch",
    "en": "English",
    "et": "Estonian",
    "fi": "Finnish",
    "fr": "French",
    "de": "German",
    "el": "Greek",
    "hu": "Hungarian",
    "it": "Italian",
    "ja": "Japanese",
    "lv": "Latvian",
    "lt": "Lithuanian",
    "pl": "Polish",
    "pt": "Portuguese",
    "ro": "Romanian",
    "ru": "Russian",
    "sk": "Slovak",
    "sl": "Slovenian",
    "es": "Spanish",
    "sv": "Swedish",
}

OUTPUT_LANG = {
    "bg": "Bulgarian",
    "zh": "Chinese (simplified)",
    "cs": "Czech",
    "da": "Danish",
    "nl": "Dutch",
    "en": "English",
    "et": "Estonian",
    "fi": "Finnish",
    "fr": "French",
    "de": "German",
    "el": "Greek",
    "hu": "Hungarian",
    "it": "Italian",
    "ja": "Japanese",
    "lv": "Latvian",
    "lt": "Lithuanian",
    "pl": "Polish",
    "pt": "Portuguese",
    "br": "Portuguese (Brazilian)",
    "ro": "Romanian",
    "ru": "Russian",
    "sk": "Slovak",
    "sl": "Slovenian",
    "es": "Spanish",
    "sv": "Swedish",
}


def translate(filepath, lang_from, lang_to, wrap_limit, delete_old):
    translator = deepl.Translator()

    lang_from = {
        "lang": lang_from,
        "description": INPUT_LANG[lang_from],
    }
    lang_to = {
        "lang": lang_to,
        "description": OUTPUT_LANG[lang_to],
    }

    if type(filepath) == str:
        filepath = [filepath]
    elif type(filepath) != list:
        raise TypeError("Filepath must be str or list")

    files = []
    for fpath in filepath:
        if os.path.isdir(fpath):
            files += glob.glob(fpath + "/**/*.srt", recursive=True)
        elif os.path.splitext(fpath)[-1].lower() == ".srt":
            files.append(fpath)

    for fpath in files:
        subs = translator.translate(
            fpath,
            lang_from,
            lang_to,
            wrap_limit,
        )

        for sub in subs:
            if len(sub.content) > wrap_limit:
                sub.content = wrap_line(sub.content, wrap_limit)

        fname = os.path.splitext(fpath)[0]
        save_srt(fname, lang_to["lang"], subs)

    translator.close()


import argparse
import easygui
import logging

from .main import INPUT_LANG, OUTPUT_LANG, translate

parser = argparse.ArgumentParser(
    description="Translates .STR files using DeepL.com",
)

parser.add_argument(
    "filepath",
    metavar="path",
    type=str,
    nargs="?",
    help="Files to convert (if directory traslates all srt files recursively)",
)

parser.add_argument(
    "-i",
    "--input-lang",
    type=str,
    default="auto",
    choices=INPUT_LANG.keys(),
    help="Language to translate from. Default: auto",
)

parser.add_argument(
    "-o",
    "--output-lang",
    type=str,
    default="es",
    choices=OUTPUT_LANG.keys(),
    help="Language to translate to. Default: es (spanish)",
)

parser.add_argument(
    "-v",
    "--verbose",
    action="store_const",
    dest="loglevel",
    const=logging.INFO,
    help="Increase output verbosity",
)

parser.add_argument(
    "-vv",
    "--debug",
    action="store_const",
    dest="loglevel",
    const=logging.DEBUG,
    default=logging.WARNING,
    help="Increase output verbosity for debugging",
)

parser.add_argument(
    "-g",
    "--show-gui",
    action="store_true",
    help="Show configuration graphical interface",
)

parser.add_argument(
    "-s",
    "--show-browser",
    action="store_true",
    help="Show browser window",
)

parser.add_argument(
    "-w",
    "--wrap-limit",
    type=int,
    default=50,
    help="Number of characters to wrap the line. Including spaces. Default: 50",
)

parser.add_argument(
    "-x", "--delete", action="store_true", help="Delete files when traslated"
)

args = parser.parse_args()
logging.basicConfig(level=args.loglevel)

if not args.filepath:
    args.filepath = easygui.diropenbox(title="Choose the folder where the .SRT are")

if not args.filepath:
    raise Exception("No folder found")

if args.show_gui:
    args.input_lang = easygui.choicebox(
        choices=[
            f"{lang} - [{description}]" for lang, description in INPUT_LANG.items()
        ]
    ).split("-")[0].strip()
    args.output_lang = easygui.choicebox(
        choices=[
            f"{lang} - [{description}]" for lang, description in OUTPUT_LANG.items()
        ]
    ).split("-")[0].strip()

if not args.show_browser:
    os.environ["MOZ_HEADLESS"] = "1"

translate(
    args.filepath,
    args.input_lang,
    args.output_lang,
    args.wrap_limit,
    args.delete,
)




from srt import Subtitle
import datetime
from srt_deepl.srt_parser import wrap_line, get_srt_portions

def test_wrap_line():
    
    text = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vestibulum orci nisi, convallis eget iaculis ac, tincidunt vel elit. tincidunt vel elit.....'
    max_char = 50
    
    wraped_line = wrap_line(text, max_char)
    
    assert len(text.split()) == len(' '.join(wraped_line.split('\n')).split())
    assert wraped_line == 'Lorem ipsum dolor sit amet, consectetur adipiscing\nelit. Vestibulum orci nisi, convallis eget iaculis\nac, tincidunt vel elit. tincidunt vel elit.....'
    assert len(wraped_line.split('\n')) == 3
    for line in wraped_line.split('\n'):
        assert len(line) <= max_char


def test_get_srt_portions():
    
    subtitles = [
        Subtitle(1, datetime.timedelta(0, 1), datetime.timedelta(0, 2), 'TEST0'),
        Subtitle(1, datetime.timedelta(0, 1), datetime.timedelta(0, 2), 'TEST1'),
        Subtitle(1, datetime.timedelta(0, 1), datetime.timedelta(0, 2), 'TEST2'),
        Subtitle(1, datetime.timedelta(0, 1), datetime.timedelta(0, 2), 'TEST3'),
    ]
    
    characters = 11
    portions = list(get_srt_portions(subtitles, characters))
    assert len(portions) == 2
    for portion in portions:
        assert len(portion) != 0
        assert sum(len(sub.content) for sub in portion) < characters
        
    characters = 10
    portions = list(get_srt_portions(subtitles, characters))
    assert len(portions) == 4
    for portion in portions:
        assert len(portion) != 0
        assert sum(len(sub.content) for sub in portion) < characters
        
    characters = 6
    portions = list(get_srt_portions(subtitles, characters))
    assert len(portions) == 4
    for portion in portions:
        assert len(portion) != 0
        assert sum(len(sub.content) for sub in portion) < characters


import srt
import logging


def open_srt(file_path):
    logging.info(f"Reading {file_path}")

    with open(file_path, "r", errors="ignore") as srt_file:
        srt_file = srt.parse(srt_file)
        subs = list(srt_file)
        subs = list(srt.sort_and_reindex(subs))

        for sub in subs:
            sub.content = srt.make_legal_content(sub.content)
            sub.content = sub.content.strip().replace("\n", " ")

        return subs


def get_srt_portions(subtitles, characters=4500):
    portion = []

    for subtitle in subtitles:
        n_char = sum(len(sub.content) for sub in portion) + len(subtitle.content)

        if n_char >= characters and len(portion) != 0:
            yield portion
            portion = []

        portion.append(subtitle)

    yield portion


def wrap_line(text, max_char):
    wraped_lines = []
    for word in text.split():
        if len(wraped_lines) != 0 and len(wraped_lines[-1]) + len(word) < max_char:
            wraped_lines[-1] += f" {word}"
        else:
            wraped_lines.append(f"{word}")

    return '\n'.join(wraped_lines)


def save_srt(file_name, lang_to, subs):

    logging.info(f"Saving {file_name}_{lang_to}")
    subs = srt.compose(subs)
    with open(f"{file_name}_{lang_to}.srt", "w") as file_out:
        file_out.write(subs)



import logging
import time
import pyperclip

from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.common.action_chains import ActionChains

from srt_deepl.elements import TextArea, Button
from srt_deepl.srt_parser import open_srt, get_srt_portions


class Translator:
    def __init__(self):
        logging.info("Opening browser")
        self.driver = webdriver.Firefox()
        self.driver.maximize_window()

        logging.info("Going to DeepL")
        self.driver.get("https://www.deepl.com/translator")

        self.actions = ActionChains(self.driver)
        self.input_lang_from = TextArea(
            self.driver,
            "CLASS_NAME",
            "lmt__source_textarea",
        )
        self.input_lang_to = TextArea(
            self.driver,
            "CLASS_NAME",
            "lmt__target_textarea",
        )

    def close(self):
        logging.info("Closing browser")
        self.driver.close()

    def choose_languages(self, lang_from, lang_to):
        Button(
            self.driver,
            "CLASS_NAME",
            "lmt__language_select--source",
        ).click()

        Button(
            self.driver,
            "XPATH",
            f"//button[@dl-test='translator-lang-option-{lang_from['lang'].lower()}'] | //button[text()='{lang_from['description']}']",
        ).click()

        Button(
            self.driver,
            "CLASS_NAME",
            "lmt__language_select--target",
        ).click()

        input_lang_button = Button(
            self.driver,
            "XPATH",
            f"//button[@dl-test='translator-lang-option-{lang_to['lang'].lower()}-{lang_to['lang'].upper()}'] | //button[text()='{lang_to['description']}']",
        ).click()

    def translate(self, file_path, lang_from, lang_to, wrap_limit):

        self.choose_languages(lang_from, lang_to)

        subs = open_srt(file_path)

        for portion in get_srt_portions(subs):
            text = [sub.content for sub in portion]
            text = "\n".join(text)

            logging.info("Copying portion of file")
            try:
                # Copy/paste to text area
                pyperclip.copy(text)
                self.input_lang_from.write((Keys.CONTROL, "v"))
            except:
                # Copies letter to letter if copy/paste fail
                self.input_lang_from.write((text))

            logging.info("Waiting for traslation to complete")
            for _ in range(60):  # Maximun number of iterations 60 seconds
                traslation = self.input_lang_to.value
                if (
                    len(traslation) != 0
                    and len(text.splitlines()) == len(traslation.splitlines())
                    and "[...]" not in traslation
                ):
                    break
                time.sleep(1)

            else:
                raise Exception(
                    """Timeout for traslating portion.\n
                    Make sure your SRT file does not contain the characters '[...]'"""
                )

            logging.info("Updating portion with traslation")
            traslation = self.input_lang_to.value.splitlines()

            for i in range(len(portion)):
                portion[i].content = traslation[i]

        return subs



from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By


class BaseElement:
    def __init__(self, driver, locate_by, locate_value):
        self.driver = driver
        locator = (getattr(By, locate_by.upper(), "ID"), locate_value)
        WebDriverWait(driver, 100).until(lambda driver: driver.find_element(*locator))
        self.element = driver.find_element(*locator)


class TextArea(BaseElement):
    def write(self, value):
        self.element.clear()
        self.element.send_keys(*value)

    @property
    def value(self):
        return self.element.get_attribute("value")


class Button(BaseElement):
    def click(self):
        try:
            can_click = getattr(self.element, "click", None)
            if callable(can_click):
                self.element.click()
        except:
            # Using javascript if usual click function does not work
            self.driver.execute_script("arguments[0].click();", self.element)



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.read().splitlines()

setup(
    name="srt_deepl",
    description="Traslate a .SRT file using DeepL and Selenium.",
    url='https://github.com/sinedie/SRT-DeepL-Translator',
    version="0.8.3",
    author="EAR",
    author_email="sinedie@protonmail.com",
    license='FREE',
    python_requires=">=3.6",
    install_requires=requirements,
    packages=find_packages(),
    keywords=["python", "srt", "deepl", "languages", "translator", "subtitles"],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
)
